classdef Shape
%Shape abstract class for optical tweezers toolbox shapes
%
% Properties
%   maxRadius     maximum distance from shape origin
%   volume        volume of shape
%
% Methods (abstract):
%   inside(shape, ...) determine if spherical point is inside shape
%
% Methods:
%   writeWavefrontObj(shape, ...) write shape to Wavefront OBJ file
%       only implemented if shape supports this action.
%   insideXyz(shape, ...) determine if Cartesian point is inside shape
%       requires inside(shape, ...) to be implemented.
%   simple(...) simplified constructor for shape-like objects.
%
% See also simple, ott.shapes.Cube, ott.shapes.TriangularMesh.
%
% This file is part of the optical tweezers toolbox.
% See LICENSE.md for information about using/distributing this file.

  properties
  end

  methods (Static)

    function shape = simple(name, parameters)
			%SIMPLE constructs a shape for a bunch of simple particles
      % This method replaces shapesurface.m from OTTv1.
      %
      % Supported shapes [parameters]:
      %   'sphere'          Sphere [ radius ]
      %   'cylinder'        z-axis aligned cylinder [ radius height ]
      %   'ellipsoid'       Ellipsoid [ a b c]
      %   'superellipsoid'  Superellipsoid [ a b c e n ]
      %   'cone-tipped-cylinder'      [ radius height cone_height ]
      %   'cube'            Cube [ width ]
      %   'axisym'          Axis-symetric particle [ rho(:) z(:) ]
      %   'stl'             load STL file [filename]
      %   'obj'             load Wavefront OBJ file [filename]

      switch lower(name)
        case 'sphere'
          shape = ott.shapes.Sphere(parameters(:));
        case 'cylinder'
          shape = ott.shapes.Cylinder(parameters(:));
        case 'ellipsoid'
          shape = ott.shapes.Ellipsoid(parameters(:));
        case 'superellipsoid'
          shape = ott.shapes.Superellipsoid(parameters(:));
        case 'cone-tipped-cylinder'

          radius = parameters(1);
          height = parameters(2);
          cone_height = parameters(3);

          z = [ height/2 + cone_height, height/2, ...
              -height/2, -height/2 - cone_height];
          rho = [ 0.0, radius, radius, 0.0 ];

          shape = ott.shapes.AxisymLerp(rho, z);
        case 'cube'
          shape = ott.shapes.Cube(parameters(:));
        case 'axisym'
          shape = ott.shapes.AxisymLerp(parameters(:));
        case 'obj'
          shape = ott.shapes.WavefrontObj(parameters(:));
        case 'stl'
          shape = ott.shapes.StlLoader(parameters(:));
        otherwise
          error('Unsupported simple particle shape');
      end
    end
  end

  properties (Dependent)
    maxRadius       % Maximum particle radius (useful for Nmax calculation)
    volume          % Volume of the particle
  end

  methods (Abstract)
    inside(shape, varargin)   % Determine if point is inside shape

    get_maxRadius(shape, varargin)    % Get max distance from origin
    get_volume(shape, varargin);      % Get shape volume
  end

  methods (Access=protected)

    function writeWavefrontObj_helper(shape, filename, verts, faces)
      % Helper to write faces and vertices to a file
      %
      % helper(filename, verts, faces)
      %   verts 3xN array of floats
      %   faces mxN array of integers (starting at 1)

      fp = fopen(filename, 'w');

      % Write documentation
      fprintf(fp, '# Shape description generated by OTT\n');

      % Write verts
      for ii = 1:size(verts, 2)
        fprintf(fp, 'v %f %f %f\n', verts(:, ii));
      end

      % Write faces
      for ii = 1:size(faces, 2)
        fprintf(fp, 'f ');
        fprintf(fp, '%d ', faces(:, ii));
        fprintf(fp, '\n');
      end

      % Close file
      fclose(fp);

    end
  end

  methods

    function r = get.maxRadius(shape)
      % Get the particle max radius
      r = shape.get_maxRadius();
    end

    function r = get.volume(shape)
      % Get the particle volume
      r = shape.get_volume();
    end

    function writeWavefrontObj(shape, filename)
      % Write representation of shape to Wavefront OBJ file
      %
      % This is a placeholder, not supported by all shape types
      error('Shape does not support writing to WavefrontOBJ file');
    end

    function surf(shape, varargin)
      % SURF generate a visualisation of the shape
      %
      % SURF() displays a visualisation of the shape in the current figure.
      %
      % SURF(..., 'surfoptions', {varargin}) specifies the options to
      % pass to the surf function.
      error('Shape does not support surf visualisation');
    end

    function varargout = voxels(shape, varargin)
      % Generate an array of xyz coordinates for voxels inside the shape
      %
      % voxels(spacing) shows a visualisation of the shape with
      % circles placed at locations on a Cartesian grid.
      %
      % xyz = voxels(spacing) returns the voxel locations.
      %
      % Optional named arguments:
      %     'plotoptions'   Options to pass to the plot3 function
      %     'visualise'     Show the visualisation (default: nargout == 0)

      p = inputParser;
      p.addOptional('spacing', shape.maxRadius/10);
      p.addParameter('plotoptions', []);
      p.addParameter('visualise', nargout == 0);
      p.addParameter('scale', 1.0);
      p.addParameter('axes', []);
      p.parse(varargin{:});

      plotoptions = p.Results.plotoptions;
      if isempty(plotoptions)
        plotoptions = {...
          'MarkerFaceColor', 'w', ...
          'MarkerEdgeColor', [.5 .5 .5], ...
          'MarkerSize', 20*p.Results.spacing/shape.maxRadius/p.Results.scale};
      end

      % Calculate range of dipoles
      numr = ceil(shape.maxRadius * p.Results.scale / p.Results.spacing);
      rrange = (-numr:numr)*p.Results.spacing;

      % Generate the voxel grid
      [xx, yy, zz] = meshgrid(rrange, rrange, rrange);

      % Determine which points are inside
      mask = shape.insideXyz(xx / p.Results.scale, yy / p.Results.scale, zz / p.Results.scale);
      xyz = [xx(mask).'; yy(mask).'; zz(mask).'];

      % Visualise the result
      if p.Results.visualise
        
        % Get the axes to use
        our_axes = p.Results.axes;
        if isempty(our_axes)
          our_axes = axes();
        end
        
        plot3(our_axes, xyz(1,:), xyz(2,:), xyz(3,:), 'o', plotoptions{:});
        axis(our_axes, 'equal');
        title(our_axes, ['spacing = ' num2str(p.Results.spacing) ...
            ', N = ' int2str(sum(mask))])
      end

      % Assign output
      if nargout ~= 0
        varargout = {xyz};
      end
    end

    function b = insideXyz(shape, x, y, z)
      % INSIDEXYZ determine if Cartesian point is inside the shape
      %
      % b = inside(shape, x, y, z) determine if the Cartesian point
      % [x, y, z] is inside the star shaped object.
      %
      % b = insideXyz(shape, xyz) as above, but using a 3xN matrix of
      % [x; y; z] positions.
      %
      % See also INSIDE.

      % Ensure the sizes match
      if nargin == 4
        x = x(:);
        y = y(:);
        z = z(:);
        [x, y, z] = ott.utils.matchsize(x, y, z);
      else
        y = x(2, :);
        z = x(3, :);
        x = x(1, :);
      end

      % Convert to spherical coordinates
      [r, t, p] = ott.utils.xyz2rtp(x, y, z);

      % Call the spherical coordinate version
      b = shape.inside(r, t, p);
    end
  end
end
